"use strict";
require("angular-layer");
angular.module("http", ["layer"]).factory("http", [
  "$q",
  "$http",
  "$window",
  "layer",
  "$compile",
  "$rootScope",
  function($q, $http, $window, layer, $compile, $rootScope) {
    var that = this;
    var deferred;
    let analyzeUrl = function analyzeUrl(url) {
      if (url === undefined) return "";
      if (url.indexOf("http") === 0) return url;
      return ($window.hapi || "") + url;
    };
    let analyzeParams = function analyzeParams(params, shift) {
      if (typeof params === "boolean" && shift) {
        params = extend(paramsObject(), shift);
      }
      if (params === undefined) params = {};
      params.timestamp = new Date().getTime();
      return params;
    };
    let post = function post(params) {
      deferred = $q.defer();
      $http({
        method: "POST",
        data: params.data || {},
        url: analyzeUrl(params.url)
      }).then(
        res => success(res, deferred),
        err => error(err, this.catche, params, deferred)
      );
      return deferred.promise;
    };
    let get = function get(params) {
      var deferred = $q.defer();
      $http({
        method: "GET",
        url: analyzeUrl(params.url),
        params: analyzeParams(params.params, params.shift)
      }).then(
        res => success(res, deferred),
        err => error(err, this.catche, params, deferred)
      );
      return deferred.promise;
    };
    let error = (err, catche = function() {}, params, deferred) => {
      if (err.status == 401 && hapi_401) {
        $window.localStorage.clear();
        var referrer = document.referrer;
        var href = $window.location.href;
        if (referrer) $window.localStorage.setItem("loginBackUrl", href);
        return hapi_401.replaceState({reload: true});
      }
      if (params.call) return params.call(err.data);
      deferred.reject(catche(err));
    };
    let success = (res, deferred) => {
      deferred.resolve(res.data);
    };
    // 获取当前url添加参数
    let getParams = function getParams(key, url) {
      var url = new URL(url ? url : $window.location.href);
      return url.searchParams.get(key);
    };
    // 给当前url添加参数
    let setParams = function setParams(key, val, url) {
      var url = new URL(url ? url : $window.location.href);
      url.searchParams.set(key, val);
      return url.href;
    };
    // 替换当前地址
    let replaceState = function replaceState(key, val, url) {
      var url = new URL(url ? url : $window.location.href);
      if (key) url = this.setParams(key, val);
      $window.history.replaceState({}, {}, url);
    };
    // 增加新地址
    let pushState = function pushState(key, val, url) {
      var url = new URL(url ? url : $window.location.href);
      if (key) url = this.setParams(key, val);
      $window.history.pushState({}, {}, url);
    };
    let extend = function extend(defaults, options) {
      var extended = {};
      var prop;
      for (prop in defaults) {
        if (Object.prototype.hasOwnProperty.call(defaults, prop)) {
          extended[prop] = defaults[prop];
        }
      }
      for (prop in options) {
        if (Object.prototype.hasOwnProperty.call(options, prop)) {
          extended[prop] = options[prop];
        }
      }
      return extended;
    };
    let paramsObject = function paramsObject() {
      var query = $window.location.href;
      query = query.substring(query.indexOf("?") + 1);
      var re = /([^&=]+)=?([^&]*)/g;
      var decodeRE = /\+/g;
      var decode = function(str) {
        return decodeURIComponent(str.replace(decodeRE, " "));
      };
      var params = {},
        e;
      while ((e = re.exec(query))) {
        var k = decode(e[1]),
          v = decode(e[2]);
        if (k.substring(k.length - 2) === "[]") {
          k = k.substring(0, k.length - 2);
          (params[k] || (params[k] = [])).push(v);
        } else params[k] = v;
      }
      var assign = function(obj, keyPath, value) {
        var lastKeyIndex = keyPath.length - 1;
        for (var i = 0; i < lastKeyIndex; ++i) {
          var key = keyPath[i];
          if (!(key in obj)) obj[key] = {};
          obj = obj[key];
        }
        obj[keyPath[lastKeyIndex]] = value;
      };
      for (var prop in params) {
        var structure = prop.split("[");
        if (structure.length > 1) {
          var levels = [];
          structure.forEach(function(item, i) {
            var key = item.replace(/[?[\]\\ ]/g, "");
            levels.push(key);
          });
          assign(params, levels, params[prop]);
          delete params[prop];
        }
      }
      return params;
    };
    return {
      get: get,
      post: post,
      error: error,
      extend: extend,
      getParams: getParams,
      setParams: setParams,
      pushState: pushState,
      replaceState: replaceState,
      paramsObject: paramsObject
    };
  }
]);
